@@ -50,8 +50,17 @@ private Q_SLOTS:
     void testSwitchToWindowFullScreen();
     void testActiveFullscreen();
     void testXdgActivation();
+    void testNoDemandAttentionWithoutActivationRequest();
+    void testXdgActivationBeforeInitialCommit();
+    void testXdgActivationBeforeMap();
     void testGlobalShortcutActivation();
     void testFocusMovesFromClosedDialogToParentWindow();
+    void testFullAreaLayerSurfaceUnderlay_data();
+    void testFullAreaLayerSurfaceUnderlay();
+    void testFullAreaLayerSurfaceOverlay_data();
+    void testFullAreaLayerSurfaceOverlay();
+    void testPartialAreaLayerSurfaceOverlay_data();
+    void testPartialAreaLayerSurfaceOverlay();
 
 private:
     void stackScreensHorizontally();
@@ -76,7 +85,9 @@ void ActivationTest::initTestCase()
 
 void ActivationTest::init()
 {
-    QVERIFY(Test::setupWaylandConnection(Test::AdditionalWaylandInterface::XdgActivation | Test::AdditionalWaylandInterface::Seat));
+    options->setFocusStealingPreventionLevel(FocusStealingPreventionLevel::Low);
+
+    QVERIFY(Test::setupWaylandConnection(Test::AdditionalWaylandInterface::XdgActivation | Test::AdditionalWaylandInterface::Seat | Test::AdditionalWaylandInterface::LayerShellV1));
 
     workspace()->setActiveOutput(QPoint(640, 512));
     input()->pointer()->warp(QPoint(640, 512));
@@ -739,6 +750,237 @@ void ActivationTest::testXdgActivation()
     QCOMPARE(workspace()->activeWindow(), windows[1]->m_window);
 }
 
+void ActivationTest::testNoDemandAttentionWithoutActivationRequest()
+{
+    // This test verifies that the demand attention state will not be set if kwin doesn't want a
+    // window to be activated and the client hasn't made an explicit activation request.
+
+    options->setFocusStealingPreventionLevel(FocusStealingPreventionLevel::High);
+
+    // Show a window
+    auto gooseWindow = std::make_unique<Test::XdgToplevelWindow>([](Test::XdgToplevel *toplevel) {
+        toplevel->set_app_id("org.kde.goose");
+    });
+    gooseWindow->show();
+    QVERIFY(gooseWindow->m_window->isActive());
+    QVERIFY(!gooseWindow->m_window->isDemandingAttention());
+
+    // Show another window.
+    auto wolfWindow = std::make_unique<Test::XdgToplevelWindow>([](Test::XdgToplevel *toplevel) {
+        toplevel->set_app_id("org.kde.wolf");
+    });
+    wolfWindow->show();
+    QVERIFY(!wolfWindow->m_window->isActive());
+    QVERIFY(!wolfWindow->m_window->isDemandingAttention());
+}
+
+static QString generateActivationToken(const Test::XdgToplevelWindow &window, const QString &appId)
+{
+    std::unique_ptr<Test::XdgActivationToken> token = Test::xdgActivation()->createToken();
+    token->set_surface(*window.m_surface);
+    token->set_serial(window.m_window->lastUsageSerial(), *Test::waylandSeat());
+    token->set_app_id(appId);
+    return token->commitAndWait();
+}
+
+void ActivationTest::testXdgActivationBeforeInitialCommit()
+{
+    // This test verifies that activation requests that are made before the initial xdg-toplevel
+    // commit are handled as expected, i.e. that the window will be eventually marked as active and
+    // that the Workspace state is not touched yet.
+
+    options->setFocusStealingPreventionLevel(FocusStealingPreventionLevel::High);
+
+    // Show a window
+    auto gooseWindow = std::make_unique<Test::XdgToplevelWindow>([](Test::XdgToplevel *toplevel) {
+        toplevel->set_app_id("org.kde.goose");
+    });
+    gooseWindow->show();
+    QVERIFY(gooseWindow->m_window->isActive());
+
+    // Click in the middle of the window so there is a proper usage serial.
+    uint32_t time = 0;
+    Test::pointerMotion(gooseWindow->m_window->frameGeometry().center(), time++);
+    Test::pointerButtonPressed(BTN_LEFT, time++);
+    Test::pointerButtonReleased(BTN_LEFT, time++);
+
+    const QString invalidActivationToken = QStringLiteral("duckity duck");
+    const QString activationToken = generateActivationToken(*gooseWindow, QStringLiteral("org.kde.wolf"));
+
+    {
+        std::unique_ptr<KWayland::Client::Surface> duckSurface(Test::createSurface());
+        std::unique_ptr<Test::XdgToplevel> duckShellSurface(Test::createXdgToplevelSurface(duckSurface.get(), Test::CreationSetup::CreateOnly));
+        QSignalSpy toplevelConfigureRequestedSpy(duckShellSurface.get(), &Test::XdgToplevel::configureRequested);
+        QSignalSpy surfaceConfigureRequestedSpy(duckShellSurface->xdgSurface(), &Test::XdgSurface::configureRequested);
+
+        // Wait for the compositor side window to be created.
+        QSignalSpy windowCreatedSpy(waylandServer(), &WaylandServer::windowCreated);
+        QVERIFY(windowCreatedSpy.wait());
+        Window *duckWindow = windowCreatedSpy.last().at(0).value<Window *>();
+
+        // Attempt to activate the window, waylandSync() is needed to make sure that the compositor
+        // side has processed the activation request before we check the compositor state.
+        //
+        // The activation token should only be cached, no other activation related state should be set.
+        Test::xdgActivation()->activate(invalidActivationToken, *duckSurface);
+        QVERIFY(Test::waylandSync());
+        QCOMPARE(duckWindow->activationToken(), invalidActivationToken);
+        QVERIFY(!duckWindow->isActive());
+        QVERIFY(!duckWindow->isDemandingAttention());
+        QVERIFY(!workspace()->stackingOrder().contains(duckWindow));
+
+        // Commit the initial state.
+        duckShellSurface->set_app_id(QStringLiteral("org.kde.duck"));
+        duckSurface->commit(KWayland::Client::Surface::CommitFlag::None);
+        QVERIFY(surfaceConfigureRequestedSpy.wait());
+
+        // Map the window.
+        QCOMPARE(toplevelConfigureRequestedSpy.last().at(0).value<QSize>(), QSize(0, 0));
+        duckShellSurface->xdgSurface()->ack_configure(surfaceConfigureRequestedSpy.last().at(0).value<quint32>());
+        Test::renderAndWaitForShown(duckSurface.get(), QSize(800, 600), Qt::blue);
+
+        // The window should not be activate because of the invalid activation token.
+        QVERIFY(!duckWindow->isActive());
+        QVERIFY(duckWindow->isDemandingAttention());
+    }
+
+    {
+        std::unique_ptr<KWayland::Client::Surface> wolfSurface(Test::createSurface());
+        std::unique_ptr<Test::XdgToplevel> wolfShellSurface(Test::createXdgToplevelSurface(wolfSurface.get(), Test::CreationSetup::CreateOnly));
+        QSignalSpy toplevelConfigureRequestedSpy(wolfShellSurface.get(), &Test::XdgToplevel::configureRequested);
+        QSignalSpy surfaceConfigureRequestedSpy(wolfShellSurface->xdgSurface(), &Test::XdgSurface::configureRequested);
+
+        // Wait for the compositor side window to be created.
+        QSignalSpy windowCreatedSpy(waylandServer(), &WaylandServer::windowCreated);
+        QVERIFY(windowCreatedSpy.wait());
+        Window *wolfWindow = windowCreatedSpy.last().at(0).value<Window *>();
+
+        // Attempt to activate the window, waylandSync() is needed to make sure that the compositor
+        // side has processed the activation request before we check the compositor state.
+        //
+        // The activation token should only be cached, no other activation related state should be set.
+        Test::xdgActivation()->activate(activationToken, *wolfSurface);
+        QVERIFY(Test::waylandSync());
+        QCOMPARE(wolfWindow->activationToken(), activationToken);
+        QVERIFY(!wolfWindow->isActive());
+        QVERIFY(!wolfWindow->isDemandingAttention());
+        QVERIFY(!workspace()->stackingOrder().contains(wolfWindow));
+
+        // Commit the initial state.
+        wolfShellSurface->set_app_id(QStringLiteral("org.kde.wolf"));
+        wolfSurface->commit(KWayland::Client::Surface::CommitFlag::None);
+        QVERIFY(surfaceConfigureRequestedSpy.wait());
+
+        // Map the window.
+        QCOMPARE(toplevelConfigureRequestedSpy.last().at(0).value<QSize>(), QSize(0, 0));
+        wolfShellSurface->xdgSurface()->ack_configure(surfaceConfigureRequestedSpy.last().at(0).value<quint32>());
+        Test::renderAndWaitForShown(wolfSurface.get(), QSize(800, 600), Qt::blue);
+
+        // The window should be activated as expected.
+        QVERIFY(wolfWindow->isActive());
+        QVERIFY(!wolfWindow->isDemandingAttention());
+    }
+}
+
+void ActivationTest::testXdgActivationBeforeMap()
+{
+    // This test verifies that activation requests that are made before the xdg-toplevel surface
+    // is mapped are handled as expected, i.e. that the window will be eventually marked as active
+    // and that the Workspace state is not touched yet.
+
+    options->setFocusStealingPreventionLevel(FocusStealingPreventionLevel::High);
+
+    // Show a window
+    auto gooseWindow = std::make_unique<Test::XdgToplevelWindow>([](Test::XdgToplevel *toplevel) {
+        toplevel->set_app_id("org.kde.goose");
+    });
+    gooseWindow->show();
+    QVERIFY(gooseWindow->m_window->isActive());
+
+    // Click in the middle of the window so there is a proper usage serial.
+    uint32_t time = 0;
+    Test::pointerMotion(gooseWindow->m_window->frameGeometry().center(), time++);
+    Test::pointerButtonPressed(BTN_LEFT, time++);
+    Test::pointerButtonReleased(BTN_LEFT, time++);
+
+    const QString invalidActivationToken = QStringLiteral("duckity duck");
+    const QString activationToken = generateActivationToken(*gooseWindow, QStringLiteral("org.kde.wolf"));
+
+    {
+        std::unique_ptr<KWayland::Client::Surface> duckSurface(Test::createSurface());
+        std::unique_ptr<Test::XdgToplevel> duckShellSurface(Test::createXdgToplevelSurface(duckSurface.get(), Test::CreationSetup::CreateOnly));
+        QSignalSpy toplevelConfigureRequestedSpy(duckShellSurface.get(), &Test::XdgToplevel::configureRequested);
+        QSignalSpy surfaceConfigureRequestedSpy(duckShellSurface->xdgSurface(), &Test::XdgSurface::configureRequested);
+
+        // Wait for the compositor side window to be created.
+        QSignalSpy windowCreatedSpy(waylandServer(), &WaylandServer::windowCreated);
+        QVERIFY(windowCreatedSpy.wait());
+        Window *duckWindow = windowCreatedSpy.last().at(0).value<Window *>();
+
+        // Commit the initial state.
+        duckShellSurface->set_app_id(QStringLiteral("org.kde.duck"));
+        duckSurface->commit(KWayland::Client::Surface::CommitFlag::None);
+        QVERIFY(surfaceConfigureRequestedSpy.wait());
+
+        // Attempt to activate the window, waylandSync() is needed to make sure that the compositor
+        // side has processed the activation request before we check the compositor state.
+        //
+        // The activation token should only be cached, no other activation related state should be set.
+        Test::xdgActivation()->activate(invalidActivationToken, *duckSurface);
+        QVERIFY(Test::waylandSync());
+        QCOMPARE(duckWindow->activationToken(), invalidActivationToken);
+        QVERIFY(!duckWindow->isActive());
+        QVERIFY(!duckWindow->isDemandingAttention());
+        QVERIFY(!workspace()->stackingOrder().contains(duckWindow));
+
+        // Map the window.
+        QCOMPARE(toplevelConfigureRequestedSpy.last().at(0).value<QSize>(), QSize(0, 0));
+        duckShellSurface->xdgSurface()->ack_configure(surfaceConfigureRequestedSpy.last().at(0).value<quint32>());
+        Test::renderAndWaitForShown(duckSurface.get(), QSize(800, 600), Qt::blue);
+
+        // The window should not be activate because of the invalid activation token.
+        QVERIFY(!duckWindow->isActive());
+        QVERIFY(duckWindow->isDemandingAttention());
+    }
+
+    {
+        std::unique_ptr<KWayland::Client::Surface> wolfSurface(Test::createSurface());
+        std::unique_ptr<Test::XdgToplevel> wolfShellSurface(Test::createXdgToplevelSurface(wolfSurface.get(), Test::CreationSetup::CreateOnly));
+        QSignalSpy toplevelConfigureRequestedSpy(wolfShellSurface.get(), &Test::XdgToplevel::configureRequested);
+        QSignalSpy surfaceConfigureRequestedSpy(wolfShellSurface->xdgSurface(), &Test::XdgSurface::configureRequested);
+
+        // Wait for the compositor side window to be created.
+        QSignalSpy windowCreatedSpy(waylandServer(), &WaylandServer::windowCreated);
+        QVERIFY(windowCreatedSpy.wait());
+        Window *wolfWindow = windowCreatedSpy.last().at(0).value<Window *>();
+
+        // Commit the initial state.
+        wolfShellSurface->set_app_id(QStringLiteral("org.kde.wolf"));
+        wolfSurface->commit(KWayland::Client::Surface::CommitFlag::None);
+        QVERIFY(surfaceConfigureRequestedSpy.wait());
+
+        // Attempt to activate the window, waylandSync() is needed to make sure that the compositor
+        // side has processed the activation request before we check the compositor state.
+        //
+        // The activation token should only be cached, no other activation related state should be set.
+        Test::xdgActivation()->activate(activationToken, *wolfSurface);
+        QVERIFY(Test::waylandSync());
+        QCOMPARE(wolfWindow->activationToken(), activationToken);
+        QVERIFY(!wolfWindow->isActive());
+        QVERIFY(!wolfWindow->isDemandingAttention());
+        QVERIFY(!workspace()->stackingOrder().contains(wolfWindow));
+
+        // Map the window.
+        QCOMPARE(toplevelConfigureRequestedSpy.last().at(0).value<QSize>(), QSize(0, 0));
+        wolfShellSurface->xdgSurface()->ack_configure(surfaceConfigureRequestedSpy.last().at(0).value<quint32>());
+        Test::renderAndWaitForShown(wolfSurface.get(), QSize(800, 600), Qt::blue);
+
+        // The window should be activated as expected.
+        QVERIFY(wolfWindow->isActive());
+        QVERIFY(!wolfWindow->isDemandingAttention());
+    }
+}
+
 class TokenSpy : public InputEventSpy
 {
 public:
@@ -825,6 +1067,119 @@ void ActivationTest::testFocusMovesFromClosedDialogToParentWindow()
     QCOMPARE(windowActivatedSpy.count(), 3);
     QCOMPARE(windowActivatedSpy.last().at(0).value<Window *>(), window);
 }
+
+void ActivationTest::testFullAreaLayerSurfaceUnderlay_data()
+{
+    QTest::addColumn<Test::LayerShellV1::layer>("layer");
+
+    QTest::addRow("background") << Test::LayerShellV1::layer_background;
+    QTest::addRow("bottom") << Test::LayerShellV1::layer_bottom;
+}
+
+void ActivationTest::testFullAreaLayerSurfaceUnderlay()
+{
+    // This test verifies that an underlay layer shell surface that covers the whole screen will not
+    // be activated without a valid activation token by accident.
+
+    options->setFocusStealingPreventionLevel(FocusStealingPreventionLevel::High);
+
+    // Show a regular window.
+    Test::XdgToplevelWindow window;
+    QVERIFY(window.show());
+    QCOMPARE(workspace()->activeWindow(), window.m_window);
+
+    // Show a fullscreen layer shell surface.
+    QFETCH(Test::LayerShellV1::layer, layer);
+    std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
+    std::unique_ptr<Test::LayerSurfaceV1> shellSurface(Test::createLayerSurfaceV1(surface.get(), QStringLiteral("test")));
+    shellSurface->set_layer(layer);
+    shellSurface->set_anchor(Test::LayerSurfaceV1::anchor_top | Test::LayerSurfaceV1::anchor_right | Test::LayerSurfaceV1::anchor_bottom | Test::LayerSurfaceV1::anchor_left);
+    shellSurface->set_size(0, 0);
+    shellSurface->set_keyboard_interactivity(1);
+    surface->commit(KWayland::Client::Surface::CommitFlag::None);
+
+    QSignalSpy configureRequestedSpy(shellSurface.get(), &Test::LayerSurfaceV1::configureRequested);
+    QVERIFY(configureRequestedSpy.wait());
+    const QSize requestedSize = configureRequestedSpy.last().at(1).toSize();
+    shellSurface->ack_configure(configureRequestedSpy.last().at(0).toUInt());
+    Test::renderAndWaitForShown(surface.get(), requestedSize, Qt::red);
+    QCOMPARE(workspace()->activeWindow(), window.m_window);
+}
+
+void ActivationTest::testFullAreaLayerSurfaceOverlay_data()
+{
+    QTest::addColumn<Test::LayerShellV1::layer>("layer");
+
+    QTest::addRow("top") << Test::LayerShellV1::layer_top;
+    QTest::addRow("overlay") << Test::LayerShellV1::layer_overlay;
+}
+
+void ActivationTest::testFullAreaLayerSurfaceOverlay()
+{
+    // This test verifies that an overlay layer shell surface that covers the whole screen will be
+    // activated even without a valid activation token (because it will cover the active window).
+
+    options->setFocusStealingPreventionLevel(FocusStealingPreventionLevel::High);
+
+    // Show a regular window.
+    Test::XdgToplevelWindow window;
+    QVERIFY(window.show());
+    QCOMPARE(workspace()->activeWindow(), window.m_window);
+
+    // Show a fullscreen layer shell surface.
+    QFETCH(Test::LayerShellV1::layer, layer);
+    std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
+    std::unique_ptr<Test::LayerSurfaceV1> shellSurface(Test::createLayerSurfaceV1(surface.get(), QStringLiteral("test")));
+    shellSurface->set_layer(layer);
+    shellSurface->set_anchor(Test::LayerSurfaceV1::anchor_top | Test::LayerSurfaceV1::anchor_right | Test::LayerSurfaceV1::anchor_bottom | Test::LayerSurfaceV1::anchor_left);
+    shellSurface->set_size(0, 0);
+    shellSurface->set_keyboard_interactivity(1);
+    surface->commit(KWayland::Client::Surface::CommitFlag::None);
+
+    QSignalSpy configureRequestedSpy(shellSurface.get(), &Test::LayerSurfaceV1::configureRequested);
+    QVERIFY(configureRequestedSpy.wait());
+    const QSize requestedSize = configureRequestedSpy.last().at(1).toSize();
+    shellSurface->ack_configure(configureRequestedSpy.last().at(0).toUInt());
+    Window *overlayWindow = Test::renderAndWaitForShown(surface.get(), requestedSize, Qt::red);
+    QCOMPARE(workspace()->activeWindow(), overlayWindow);
+}
+
+void ActivationTest::testPartialAreaLayerSurfaceOverlay_data()
+{
+    QTest::addColumn<Test::LayerShellV1::layer>("layer");
+
+    QTest::addRow("top") << Test::LayerShellV1::layer_top;
+    QTest::addRow("overlay") << Test::LayerShellV1::layer_overlay;
+}
+
+void ActivationTest::testPartialAreaLayerSurfaceOverlay()
+{
+    // This test verifies that an overlay layer shell surface that partially covers the screen will
+    // not be activated without an activation token by accident.
+
+    options->setFocusStealingPreventionLevel(FocusStealingPreventionLevel::High);
+
+    // Show a regular window.
+    Test::XdgToplevelWindow window;
+    QVERIFY(window.show());
+    QCOMPARE(workspace()->activeWindow(), window.m_window);
+
+    // Show a fullscreen layer shell surface.
+    QFETCH(Test::LayerShellV1::layer, layer);
+    std::unique_ptr<KWayland::Client::Surface> surface(Test::createSurface());
+    std::unique_ptr<Test::LayerSurfaceV1> shellSurface(Test::createLayerSurfaceV1(surface.get(), QStringLiteral("test")));
+    shellSurface->set_layer(layer);
+    shellSurface->set_size(1000, 1000);
+    shellSurface->set_keyboard_interactivity(1);
+    surface->commit(KWayland::Client::Surface::CommitFlag::None);
+
+    QSignalSpy configureRequestedSpy(shellSurface.get(), &Test::LayerSurfaceV1::configureRequested);
+    QVERIFY(configureRequestedSpy.wait());
+    const QSize requestedSize = configureRequestedSpy.last().at(1).toSize();
+    shellSurface->ack_configure(configureRequestedSpy.last().at(0).toUInt());
+    Test::renderAndWaitForShown(surface.get(), requestedSize, Qt::red);
+    QCOMPARE(workspace()->activeWindow(), window.m_window);
+}
 }
 
 WAYLANDTEST_MAIN(KWin::ActivationTest)
