@@ -138,6 +138,7 @@ private Q_SLOTS:
     void testSettingRestoration();
     void testSettingRestoration_initialParsingFailure();
     void testSettingRestoration_replacedMode();
+    void testCursorRestoration();
 
     void testEvacuateTiledWindowFromRemovedOutput_data();
     void testEvacuateTiledWindowFromRemovedOutput();
@@ -1489,6 +1490,30 @@ void OutputChangesTest::testGenerateConfigs_data()
                .modes = {ModeInfo(QSize(3840, 2160), 120000, OutputMode::Flag::Preferred)},
            }
         << ModeInfo(QSize(3840, 2160), 120000, OutputMode::Flag::Preferred) << 2.0 << true;
+
+    QTest::addRow("Acer 24 CB242Ybmiprx")
+        << DeviceType::Desktop
+        << Test::OutputInfo{
+               .geometry = QRect(),
+               .internal = false,
+               .physicalSizeInMM = QSize(527, 296),
+               .modes = {
+                   ModeInfo(QSize(1920, 1080), 60000, OutputMode::Flag::Preferred),
+                   ModeInfo(QSize(1920, 1080), 75000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1920, 1080), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1920, 1080), 50000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1680, 1050), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1280, 1024), 75000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1280, 1024), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1440, 900), 50000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1280, 960), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1920, 540), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1280, 800), 60000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1152, 864), 75000, OutputMode::Flags{}),
+                   ModeInfo(QSize(1280, 720), 60000, OutputMode::Flags{}),
+               },
+           }
+        << ModeInfo(QSize(1920, 1080), 75000, OutputMode::Flags{}) << 1.0 << true;
 }
 
 void OutputChangesTest::testGenerateConfigs()
@@ -2017,6 +2042,43 @@ void OutputChangesTest::testSettingRestoration_replacedMode()
     QCOMPARE(output->currentMode(), output->modes()[2]);
 }
 
+void OutputChangesTest::testCursorRestoration()
+{
+    // This test verifies that the cursor gets put back to its original position
+    // when output changes happen, even with edge barriers enabled
+    options->setEdgeBarrier(100);
+
+    Test::setOutputConfig({
+        Test::OutputInfo{
+            .geometry = Rect(0, 0, 2880, 1920),
+            .scale = 1.6,
+            .connectorName = QStringLiteral("eDP-1"),
+        },
+    });
+
+    BackendOutput *tmp = kwinApp()->outputBackend()->createVirtualOutput("DP-1", "", QSize(5120, 1440), 1.0);
+
+    auto outputs = kwinApp()->outputBackend()->outputs();
+
+    OutputConfiguration config;
+    config.changeSet(outputs[0])->pos = QPoint(1691, 1440);
+    config.changeSet(outputs[1])->pos = QPoint(0, 0);
+    QCOMPARE(workspace()->applyOutputConfiguration(config), OutputConfigurationError::None);
+
+    input()->pointer()->warp(outputs[0]->position() + QPoint(1500, 1000));
+
+    // if an unrelated output is removed, the cursor should stay where it was
+    // relative to the output it's on
+    kwinApp()->outputBackend()->removeVirtualOutput(tmp);
+    outputs = kwinApp()->outputBackend()->outputs();
+    QCOMPARE(input()->pointer()->pos(), outputs[0]->position() + QPoint(1500, 1000));
+
+    // same when it's added back
+    tmp = kwinApp()->outputBackend()->createVirtualOutput("DP-1", "", QSize(5120, 1440), 1.0);
+    outputs = kwinApp()->outputBackend()->outputs();
+    QCOMPARE(input()->pointer()->pos(), outputs[0]->position() + QPoint(1500, 1000));
+}
+
 void OutputChangesTest::testEvacuateTiledWindowFromRemovedOutput_data()
 {
     QTest::addColumn<QuickTileFlag>("tileMode");
