@@ -17,6 +17,7 @@
 #include "drm_output.h"
 #include "drm_pipeline.h"
 #include "drm_virtual_egl_layer.h"
+#include "drm_virtual_output.h"
 // system
 #include <drm_fourcc.h>
 #include <gbm.h>
@@ -37,12 +38,6 @@ EglGbmBackend::EglGbmBackend(DrmBackend *drmBackend)
 EglGbmBackend::~EglGbmBackend()
 {
     m_backend->releaseBuffers();
-    const auto outputs = m_backend->outputs();
-    for (const auto output : outputs) {
-        if (auto drmOutput = dynamic_cast<DrmOutput *>(output)) {
-            drmOutput->pipeline()->setLayers(nullptr, nullptr);
-        }
-    }
     m_contexts.clear();
     cleanup();
     m_backend->setRenderBackend(nullptr);
@@ -141,46 +136,28 @@ DrmDevice *EglGbmBackend::drmDevice() const
     return gpu()->drmDevice();
 }
 
-bool EglGbmBackend::present(Output *output, const std::shared_ptr<OutputFrame> &frame)
+QList<OutputLayer *> EglGbmBackend::compatibleOutputLayers(BackendOutput *output)
 {
-    return static_cast<DrmAbstractOutput *>(output)->present(frame);
-}
-
-void EglGbmBackend::repairPresentation(Output *output)
-{
-    // read back drm properties, most likely our info is out of date somehow
-    // or we need a modeset
-    QTimer::singleShot(0, m_backend, &DrmBackend::updateOutputs);
-}
-
-OutputLayer *EglGbmBackend::primaryLayer(Output *output)
-{
-    return static_cast<DrmAbstractOutput *>(output)->primaryLayer();
-}
-
-OutputLayer *EglGbmBackend::cursorLayer(Output *output)
-{
-    return static_cast<DrmAbstractOutput *>(output)->cursorLayer();
-}
-
-std::pair<std::shared_ptr<KWin::GLTexture>, ColorDescription> EglGbmBackend::textureForOutput(Output *output) const
-{
-    const auto drmOutput = static_cast<DrmAbstractOutput *>(output);
-    if (const auto virtualLayer = dynamic_cast<VirtualEglGbmLayer *>(drmOutput->primaryLayer())) {
-        return std::make_pair(virtualLayer->texture(), virtualLayer->colorDescription());
+    if (auto virtualOutput = qobject_cast<DrmVirtualOutput *>(output)) {
+        return {virtualOutput->primaryLayer()};
+    } else {
+        return static_cast<DrmOutput *>(output)->pipeline()->gpu()->compatibleOutputLayers(output);
     }
-    const auto layer = static_cast<EglGbmLayer *>(drmOutput->primaryLayer());
-    return std::make_pair(layer->texture(), layer->colorDescription());
 }
 
-std::shared_ptr<DrmPipelineLayer> EglGbmBackend::createDrmPlaneLayer(DrmPipeline *pipeline, DrmPlane::TypeIndex type)
+std::unique_ptr<DrmPipelineLayer> EglGbmBackend::createDrmPlaneLayer(DrmPlane *plane)
 {
-    return std::make_shared<EglGbmLayer>(this, pipeline, type);
+    return std::make_unique<EglGbmLayer>(this, plane);
 }
 
-std::shared_ptr<DrmOutputLayer> EglGbmBackend::createLayer(DrmVirtualOutput *output)
+std::unique_ptr<DrmPipelineLayer> EglGbmBackend::createDrmPlaneLayer(DrmGpu *gpu, DrmPlane::TypeIndex type)
 {
-    return std::make_shared<VirtualEglGbmLayer>(this, output);
+    return std::make_unique<EglGbmLayer>(this, gpu, type);
+}
+
+std::unique_ptr<DrmOutputLayer> EglGbmBackend::createLayer(DrmVirtualOutput *output)
+{
+    return std::make_unique<VirtualEglGbmLayer>(this, output);
 }
 
 DrmGpu *EglGbmBackend::gpu() const
