@@ -32,6 +32,7 @@ ButtonsModel::ButtonsModel(QObject *parent)
                        DecorationButtonType::ContextHelp,
                        DecorationButtonType::KeepBelow,
                        DecorationButtonType::KeepAbove,
+                       DecorationButtonType::ExcludeFromCapture,
                        DecorationButtonType::Spacer,
                    }),
                    parent)
@@ -69,6 +70,8 @@ static QString buttonToName(DecorationButtonType type)
         return i18n("Keep below other windows");
     case DecorationButtonType::KeepAbove:
         return i18n("Keep above other windows");
+    case DecorationButtonType::ExcludeFromCapture:
+        return i18n("Hide from screencast");
     case DecorationButtonType::Spacer:
         return i18n("Spacer");
     default:
