@@ -14,6 +14,8 @@
 #include <QDBusConnection>
 #include <QTimer>
 
+using namespace std::chrono_literals;
+
 namespace KWin
 {
 BlendChanges::BlendChanges()
@@ -36,7 +38,7 @@ bool BlendChanges::supported()
 
 void KWin::BlendChanges::start(int delay)
 {
-    int animationDuration = animationTime(400);
+    int animationDuration = animationTime(400ms);
 
     if (!supported() || m_state != Off) {
         return;
@@ -80,10 +82,10 @@ void BlendChanges::postPaintScreen()
     effects->addRepaintFull();
 }
 
-void BlendChanges::paintWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, QRegion region, WindowPaintData &data)
+void BlendChanges::paintWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, const Region &deviceRegion, WindowPaintData &data)
 {
     data.setCrossFadeProgress(m_timeline.value());
-    effects->paintWindow(renderTarget, viewport, w, mask, region, data);
+    effects->paintWindow(renderTarget, viewport, w, mask, deviceRegion, data);
 }
 
 void BlendChanges::prePaintScreen(ScreenPrePaintData &data, std::chrono::milliseconds presentTime)
