@@ -12,6 +12,7 @@
 // own
 #include "slide.h"
 #include "core/output.h"
+#include "core/renderviewport.h"
 #include "effect/effecthandler.h"
 
 // KConfigSkeleton
@@ -41,8 +42,6 @@ SlideEffect::SlideEffect()
             this, &SlideEffect::finishedSwitching);
     connect(effects, &EffectsHandler::desktopRemoved,
             this, &SlideEffect::finishedSwitching);
-    connect(effects, &EffectsHandler::desktopMoved,
-            this, &SlideEffect::desktopMoved);
     connect(effects, &EffectsHandler::screenAdded,
             this, &SlideEffect::finishedSwitching);
     connect(effects, &EffectsHandler::screenRemoved,
@@ -53,8 +52,6 @@ SlideEffect::SlideEffect()
     connect(effects, &EffectsHandler::currentActivityChanged, this, [this]() {
         m_switchingActivity = false;
     });
-
-    m_currentPosition = effects->desktopGridCoords(effects->currentDesktop());
 }
 
 SlideEffect::~SlideEffect()
@@ -82,10 +79,10 @@ void SlideEffect::reconfigure(ReconfigureFlags)
     m_slideBackground = SlideConfig::slideBackground();
 }
 
-inline QRegion buildClipRegion(const QPoint &pos, int w, int h)
+inline Region buildClipRegion(const QPoint &pos, int w, int h)
 {
     const QSize screenSize = effects->virtualScreenSize();
-    QRegion r = QRect(pos, screenSize);
+    Region r = Rect(pos, screenSize);
     if (effects->optionRollOverDesktops()) {
         r += (r & QRect(-w, 0, w, h)).translated(w, 0); // W
         r += (r & QRect(w, 0, w, h)).translated(-w, 0); // E
@@ -103,38 +100,47 @@ inline QRegion buildClipRegion(const QPoint &pos, int w, int h)
 
 void SlideEffect::prePaintScreen(ScreenPrePaintData &data, std::chrono::milliseconds presentTime)
 {
-    std::chrono::milliseconds timeDelta = std::chrono::milliseconds::zero();
-    if (m_lastPresentTime.count()) {
-        timeDelta = presentTime - m_lastPresentTime;
-    }
-    m_lastPresentTime = presentTime;
-
-    if (m_state == State::ActiveAnimation) {
-        m_motionX.advance(timeDelta);
-        m_motionY.advance(timeDelta);
-        const QSize virtualSpaceSize = effects->virtualScreenSize();
-        m_currentPosition.setX(m_motionX.position() / virtualSpaceSize.width());
-        m_currentPosition.setY(m_motionY.position() / virtualSpaceSize.height());
-    }
-
     const QList<VirtualDesktop *> desktops = effects->desktops();
     const int w = effects->desktopGridWidth();
     const int h = effects->desktopGridHeight();
 
+    switch (m_state) {
+    case State::Inactive:
+        Q_UNREACHABLE();
+
+    case State::ActiveAnimation: {
+        std::chrono::milliseconds timeDelta = std::chrono::milliseconds::zero();
+        if (m_lastPresentTime.count()) {
+            timeDelta = presentTime - m_lastPresentTime;
+        }
+        m_lastPresentTime = presentTime;
+
+        m_motionX.advance(timeDelta);
+        m_motionY.advance(timeDelta);
+
+        const QSize virtualSpaceSize = effects->virtualScreenSize();
+        m_paintCtx.position = QPointF(m_motionX.position() / virtualSpaceSize.width(), m_motionY.position() / virtualSpaceSize.height());
+        break;
+    }
+    case State::ActiveGesture:
+        m_paintCtx.position = m_gesturePos;
+        break;
+    }
+
     // Clipping
     m_paintCtx.visibleDesktops.clear();
     m_paintCtx.visibleDesktops.reserve(4); // 4 - maximum number of visible desktops
     bool includedX = false, includedY = false;
     for (VirtualDesktop *desktop : desktops) {
         const QPoint coords = effects->desktopGridCoords(desktop);
-        if (coords.x() % w == (int)(m_currentPosition.x()) % w) {
+        if (coords.x() % w == (int)(m_paintCtx.position.x()) % w) {
             includedX = true;
-        } else if (coords.x() % w == ((int)(m_currentPosition.x()) + 1) % w) {
+        } else if (coords.x() % w == ((int)(m_paintCtx.position.x()) + 1) % w) {
             includedX = true;
         }
-        if (coords.y() % h == (int)(m_currentPosition.y()) % h) {
+        if (coords.y() % h == (int)(m_paintCtx.position.y()) % h) {
             includedY = true;
-        } else if (coords.y() % h == ((int)(m_currentPosition.y()) + 1) % h) {
+        } else if (coords.y() % h == ((int)(m_paintCtx.position.y()) + 1) % h) {
             includedY = true;
         }
 
@@ -148,13 +154,13 @@ void SlideEffect::prePaintScreen(ScreenPrePaintData &data, std::chrono::millisec
     effects->prePaintScreen(data, presentTime);
 }
 
-void SlideEffect::paintScreen(const RenderTarget &renderTarget, const RenderViewport &viewport, int mask, const QRegion &region, Output *screen)
+void SlideEffect::paintScreen(const RenderTarget &renderTarget, const RenderViewport &viewport, int mask, const Region &deviceRegion, LogicalOutput *screen)
 {
     m_paintCtx.wrap = effects->optionRollOverDesktops();
-    effects->paintScreen(renderTarget, viewport, mask, region, screen);
+    effects->paintScreen(renderTarget, viewport, mask, deviceRegion, screen);
 }
 
-QPoint SlideEffect::getDrawCoords(QPointF pos, Output *screen)
+QPoint SlideEffect::getDrawCoords(QPointF pos, LogicalOutput *screen)
 {
     QPoint c = QPoint();
     c.setX(pos.x() * (screen->geometry().width() + m_hGap));
@@ -198,27 +204,27 @@ bool SlideEffect::willBePainted(const EffectWindow *w) const
     return false;
 }
 
-void SlideEffect::prePaintWindow(EffectWindow *w, WindowPrePaintData &data, std::chrono::milliseconds presentTime)
+void SlideEffect::prePaintWindow(RenderView *view, EffectWindow *w, WindowPrePaintData &data, std::chrono::milliseconds presentTime)
 {
     data.setTransformed();
-    effects->prePaintWindow(w, data, presentTime);
+    effects->prePaintWindow(view, w, data, presentTime);
 }
 
-void SlideEffect::paintWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, QRegion region, WindowPaintData &data)
+void SlideEffect::paintWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, const Region &deviceGeometry, WindowPaintData &data)
 {
     if (!willBePainted(w)) {
         return;
     }
 
     if (!isTranslated(w)) {
-        effects->paintWindow(renderTarget, viewport, w, mask, region, data);
+        effects->paintWindow(renderTarget, viewport, w, mask, deviceGeometry, data);
         return;
     }
 
     const int gridWidth = effects->desktopGridWidth();
     const int gridHeight = effects->desktopGridHeight();
 
-    QPointF drawPosition = forcePositivePosition(m_currentPosition);
+    QPointF drawPosition = forcePositivePosition(m_paintCtx.position);
     drawPosition = m_paintCtx.wrap ? constrainToDrawableRange(drawPosition) : drawPosition;
 
     // If we're wrapping, draw the desktop in the second position.
@@ -240,17 +246,17 @@ void SlideEffect::paintWindow(const RenderTarget &renderTarget, const RenderView
             desktopTranslation = QPointF(desktopTranslation.x(), desktopTranslation.y() + gridHeight);
         }
 
-        for (Output *screen : screens) {
+        for (LogicalOutput *screen : screens) {
             QPoint drawTranslation = getDrawCoords(desktopTranslation, screen);
             data += drawTranslation;
 
-            const QRect screenArea = screen->geometry();
-            const QRect damage = screenArea.translated(drawTranslation).intersected(screenArea);
+            const Rect screenArea = screen->geometry();
+            const Rect logicalDamage = screenArea.translated(drawTranslation).intersected(screenArea);
 
             effects->paintWindow(
                 renderTarget, viewport, w, mask,
                 // Only paint the region that intersects the current screen and desktop.
-                region.intersected(damage),
+                deviceGeometry.intersected(viewport.mapToDeviceCoordinatesAligned(logicalDamage)),
                 data);
 
             // Undo the translation for the next screen. I know, it hurts me too.
@@ -296,7 +302,7 @@ bool SlideEffect::shouldElevate(const EffectWindow *w) const
  * Called AFTER the gesture is released.
  * Sets up animation to round off to the new current desktop.
  */
-void SlideEffect::startAnimation(VirtualDesktop *old, VirtualDesktop *current, EffectWindow *movingWindow)
+void SlideEffect::startAnimation(const QPointF &oldPos, VirtualDesktop *current, EffectWindow *movingWindow)
 {
     if (m_state == State::Inactive) {
         prepareSwitching();
@@ -305,7 +311,7 @@ void SlideEffect::startAnimation(VirtualDesktop *old, VirtualDesktop *current, E
     m_state = State::ActiveAnimation;
     m_movingWindow = movingWindow;
 
-    m_startPos = m_currentPosition;
+    m_startPos = oldPos;
     m_endPos = effects->desktopGridCoords(current);
     if (effects->optionRollOverDesktops()) {
         optimizePath();
@@ -316,6 +322,7 @@ void SlideEffect::startAnimation(VirtualDesktop *old, VirtualDesktop *current, E
     m_motionX.setPosition(m_startPos.x() * virtualSpaceSize.width());
     m_motionY.setAnchor(m_endPos.y() * virtualSpaceSize.height());
     m_motionY.setPosition(m_startPos.y() * virtualSpaceSize.height());
+    m_lastPresentTime = std::chrono::milliseconds::zero();
 
     effects->setActiveFullScreenEffect(this);
     effects->addRepaintFull();
@@ -359,19 +366,33 @@ void SlideEffect::finishedSwitching()
     m_windowData.clear();
     m_movingWindow = nullptr;
     m_state = State::Inactive;
-    m_lastPresentTime = std::chrono::milliseconds::zero();
     effects->setActiveFullScreenEffect(nullptr);
-    m_currentPosition = effects->desktopGridCoords(effects->currentDesktop());
 }
 
 void SlideEffect::desktopChanged(VirtualDesktop *old, VirtualDesktop *current, EffectWindow *with)
 {
     if (m_switchingActivity || (effects->hasActiveFullScreenEffect() && effects->activeFullScreenEffect() != this)) {
-        m_currentPosition = effects->desktopGridCoords(effects->currentDesktop());
         return;
     }
 
-    startAnimation(old, current, with);
+    QPointF previousPos;
+    switch (m_state) {
+    case State::Inactive:
+        previousPos = effects->desktopGridCoords(old);
+        break;
+
+    case State::ActiveAnimation: {
+        const QSize virtualSpaceSize = effects->virtualScreenSize();
+        previousPos = QPointF(m_motionX.position() / virtualSpaceSize.width(), m_motionY.position() / virtualSpaceSize.height());
+        break;
+    }
+
+    case State::ActiveGesture:
+        previousPos = m_gesturePos;
+        break;
+    }
+
+    startAnimation(previousPos, current, with);
 }
 
 void SlideEffect::desktopChanging(VirtualDesktop *old, QPointF desktopOffset, EffectWindow *with)
@@ -389,13 +410,13 @@ void SlideEffect::desktopChanging(VirtualDesktop *old, QPointF desktopOffset, Ef
 
     // Find desktop position based on animationDelta
     QPoint gridPos = effects->desktopGridCoords(old);
-    m_currentPosition.setX(gridPos.x() + desktopOffset.x());
-    m_currentPosition.setY(gridPos.y() + desktopOffset.y());
+    m_gesturePos.setX(gridPos.x() + desktopOffset.x());
+    m_gesturePos.setY(gridPos.y() + desktopOffset.y());
 
     if (effects->optionRollOverDesktops()) {
-        m_currentPosition = forcePositivePosition(m_currentPosition);
+        m_gesturePos = forcePositivePosition(m_gesturePos);
     } else {
-        m_currentPosition = moveInsideDesktopGrid(m_currentPosition);
+        m_gesturePos = moveInsideDesktopGrid(m_gesturePos);
     }
 
     effects->setActiveFullScreenEffect(this);
@@ -407,15 +428,10 @@ void SlideEffect::desktopChangingCancelled()
     // If the fingers have been lifted and the current desktop didn't change, start animation
     // to move back to the original virtual desktop.
     if (effects->activeFullScreenEffect() == this) {
-        startAnimation(effects->currentDesktop(), effects->currentDesktop(), nullptr);
+        startAnimation(m_gesturePos, effects->currentDesktop(), nullptr);
     }
 }
 
-void SlideEffect::desktopMoved()
-{
-    m_currentPosition = effects->desktopGridCoords(effects->currentDesktop());
-}
-
 QPointF SlideEffect::moveInsideDesktopGrid(QPointF p)
 {
     if (p.x() < 0) {
