@@ -7,8 +7,8 @@
 #pragma once
 
 #include "opengl/glutils.h"
-#include "platformsupport/scenes/opengl/openglsurfacetexture.h"
 #include "scene/itemrenderer.h"
+#include "scene/surfaceitem.h"
 
 #include <unordered_set>
 
@@ -22,16 +22,28 @@ class KWIN_EXPORT ItemRendererOpenGL : public ItemRenderer
 public:
     struct RenderNode
     {
-        std::variant<GLTexture *, OpenGLSurfaceContents> texture;
+        ShaderTraits traits;
+        QVarLengthArray<GLTexture *, 4> textures;
         RenderGeometry geometry;
         QMatrix4x4 transformMatrix;
         int firstVertex = 0;
         int vertexCount = 0;
         qreal opacity = 1;
         bool hasAlpha = false;
-        ColorDescription colorDescription;
+        std::shared_ptr<ColorDescription> colorDescription;
         RenderingIntent renderingIntent;
         std::shared_ptr<SyncReleasePoint> bufferReleasePoint;
+        QVector4D box;
+        QVector4D borderRadius;
+        int borderThickness = 0;
+        QColor borderColor;
+        bool paintHole = false;
+    };
+
+    struct RenderCorner
+    {
+        RectF box;
+        BorderRadius radius;
     };
 
     struct RenderContext
@@ -39,11 +51,14 @@ public:
         QList<RenderNode> renderNodes;
         QStack<QMatrix4x4> transformStack;
         QStack<qreal> opacityStack;
+        QStack<RenderCorner> cornerStack;
         const QMatrix4x4 projectionMatrix;
         const QMatrix4x4 rootTransform;
-        const QRegion clip;
+        const Region deviceClip;
         const bool hardwareClipping;
         const qreal renderTargetScale;
+        const QPointF viewportOrigin;
+        const QPoint renderOffset;
     };
 
     ItemRendererOpenGL(EglDisplay *eglDisplay);
@@ -51,16 +66,16 @@ public:
     void beginFrame(const RenderTarget &renderTarget, const RenderViewport &viewport) override;
     void endFrame() override;
 
-    void renderBackground(const RenderTarget &renderTarget, const RenderViewport &viewport, const QRegion &region) override;
-    void renderItem(const RenderTarget &renderTarget, const RenderViewport &viewport, Item *item, int mask, const QRegion &region, const WindowPaintData &data) override;
+    void renderBackground(const RenderTarget &renderTarget, const RenderViewport &viewport, const Region &deviceRegion) override;
+    void renderItem(const RenderTarget &renderTarget, const RenderViewport &viewport, Item *item, int mask, const Region &deviceRegion, const WindowPaintData &data, const std::function<bool(Item *)> &filter, const std::function<bool(Item *)> &holeFilter) override;
 
     std::unique_ptr<ImageItem> createImageItem(Item *parent = nullptr) override;
 
 private:
     QVector4D modulate(float opacity, float brightness) const;
     void setBlendEnabled(bool enabled);
-    void createRenderNode(Item *item, RenderContext *context);
-    void visualizeFractional(const RenderViewport &viewport, const QRegion &region, const RenderContext &renderContext);
+    void createRenderNode(Item *item, RenderContext *context, const std::function<bool(Item *)> &filter, const std::function<bool(Item *)> &holeFilter);
+    void visualizeFractional(const RenderViewport &viewport, const Region &logicalRegion, const RenderContext &renderContext);
 
     bool m_blendingEnabled = false;
     EglDisplay *const m_eglDisplay;
