@@ -183,17 +183,26 @@ void DataDeviceInterface::drop()
     d->send_drop();
 }
 
-static DataDeviceManagerInterface::DnDAction chooseDndAction(AbstractDataSource *source, DataOfferInterface *offer, Qt::KeyboardModifiers keyboardModifiers)
+static DnDAction chooseDndAction(AbstractDataSource *source, DataOfferInterface *offer, Qt::KeyboardModifiers keyboardModifiers)
 {
-    // first compositor picks an action if modifiers are pressed and it's supported both sides
+    // first the compositor picks an action forced by the data source
+    if (const auto exclusiveAction = source->exclusiveAction()) {
+        if (source->supportedDragAndDropActions().testFlag(*exclusiveAction) && offer->supportedDragAndDropActions().has_value() && offer->supportedDragAndDropActions()->testFlag(*exclusiveAction)) {
+            return *exclusiveAction;
+        } else {
+            return DnDAction::None;
+        }
+    }
+
+    // then the compositor picks an action if modifiers are pressed and it's supported both sides
     if (keyboardModifiers.testFlag(Qt::ControlModifier)) {
-        if (source->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Copy) && offer->supportedDragAndDropActions().has_value() && offer->supportedDragAndDropActions()->testFlag(DataDeviceManagerInterface::DnDAction::Copy)) {
-            return DataDeviceManagerInterface::DnDAction::Copy;
+        if (source->supportedDragAndDropActions().testFlag(DnDAction::Copy) && offer->supportedDragAndDropActions().has_value() && offer->supportedDragAndDropActions()->testFlag(DnDAction::Copy)) {
+            return DnDAction::Copy;
         }
     }
     if (keyboardModifiers.testFlag(Qt::ShiftModifier)) {
-        if (source->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Move) && offer->supportedDragAndDropActions().has_value() && offer->supportedDragAndDropActions()->testFlag(DataDeviceManagerInterface::DnDAction::Move)) {
-            return DataDeviceManagerInterface::DnDAction::Move;
+        if (source->supportedDragAndDropActions().testFlag(DnDAction::Move) && offer->supportedDragAndDropActions().has_value() && offer->supportedDragAndDropActions()->testFlag(DnDAction::Move)) {
+            return DnDAction::Move;
         }
     }
 
@@ -206,14 +215,14 @@ static DataDeviceManagerInterface::DnDAction chooseDndAction(AbstractDataSource
 
     // finally pick something everyone supports in a deterministic fashion
     if (offer->supportedDragAndDropActions().has_value()) {
-        for (const DataDeviceManagerInterface::DnDAction action : {DataDeviceManagerInterface::DnDAction::Copy, DataDeviceManagerInterface::DnDAction::Move, DataDeviceManagerInterface::DnDAction::Ask}) {
+        for (const DnDAction action : {DnDAction::Copy, DnDAction::Move, DnDAction::Ask}) {
             if (source->supportedDragAndDropActions().testFlag(action) && offer->supportedDragAndDropActions()->testFlag(action)) {
                 return action;
             }
         }
     }
 
-    return DataDeviceManagerInterface::DnDAction::None;
+    return DnDAction::None;
 }
 
 void DataDeviceInterface::updateDragTarget(SurfaceInterface *surface, const QPointF &position, quint32 serial)
@@ -237,10 +246,11 @@ void DataDeviceInterface::updateDragTarget(SurfaceInterface *surface, const QPoi
         // Keep the data offer alive so the target client can retrieve data after a drop. The
         // data offer will be destroyed after the target client has finished using it.
         if (dragSource && dragSource->isDropPerformed()) {
-            if (dragSource->selectedDndAction() != DataDeviceManagerInterface::DnDAction::Ask) {
+            if (dragSource->selectedDndAction() != DnDAction::Ask) {
                 disconnect(d->drag.sourceActionConnection);
                 disconnect(d->drag.targetActionConnection);
                 disconnect(d->drag.keyboardModifiersConnection);
+                disconnect(d->drag.exclusiveActionConnection);
             }
         } else {
             delete d->drag.offer;
@@ -251,6 +261,7 @@ void DataDeviceInterface::updateDragTarget(SurfaceInterface *surface, const QPoi
         d->drag.sourceActionConnection = QMetaObject::Connection();
         d->drag.targetActionConnection = QMetaObject::Connection();
         d->drag.keyboardModifiersConnection = QMetaObject::Connection();
+        d->drag.exclusiveActionConnection = QMetaObject::Connection();
     }
 
     if (!dragSource) {
@@ -259,7 +270,7 @@ void DataDeviceInterface::updateDragTarget(SurfaceInterface *surface, const QPoi
 
     if (!surface) {
         if (!dragSource->isDropPerformed()) {
-            dragSource->dndAction(DataDeviceManagerInterface::DnDAction::None);
+            dragSource->dndAction(DnDAction::None);
         }
         return;
     }
@@ -289,7 +300,7 @@ void DataDeviceInterface::updateDragTarget(SurfaceInterface *surface, const QPoi
                 keyboardModifiers = dragSource->keyboardModifiers();
             }
 
-            const DataDeviceManagerInterface::DnDAction action = chooseDndAction(dragSource, offer, keyboardModifiers);
+            const DnDAction action = chooseDndAction(dragSource, offer, keyboardModifiers);
             offer->dndAction(action);
             dragSource->dndAction(action);
         };
@@ -297,6 +308,7 @@ void DataDeviceInterface::updateDragTarget(SurfaceInterface *surface, const QPoi
         d->drag.targetActionConnection = connect(d->drag.offer, &DataOfferInterface::dragAndDropActionsChanged, dragSource, matchOffers);
         d->drag.sourceActionConnection = connect(dragSource, &AbstractDataSource::supportedDragAndDropActionsChanged, d->drag.offer, matchOffers);
         d->drag.keyboardModifiersConnection = connect(dragSource, &AbstractDataSource::keyboardModifiersChanged, d->drag.offer, matchOffers);
+        d->drag.exclusiveActionConnection = connect(dragSource, &AbstractDataSource::exclusiveActionChanged, d->drag.offer, matchOffers);
     }
 
     const QPointF pos = d->seat->dragSurfaceTransformation().map(position);
